
import logging

import predix.app
import predix.admin.uaa
import predix.admin.acs
import predix.admin.asset
import predix.admin.weather
import predix.admin.cf.spaces
import predix.admin.blobstore
import predix.admin.timeseries
import predix.admin.logstash


class Manifest(predix.app.Manifest):
    """
    Extends Application Manifest with administrative
    functions for creating and configuring services.
    """
    def __init__(self, *args, **kwargs):
        super(Manifest, self).__init__(*args, **kwargs)

        self.space = predix.admin.cf.spaces.Space()

        self.supported = {
            'predix-uaa': predix.admin.uaa.UserAccountAuthentication,
            'predix-acs': predix.admin.acs.AccessControl,
            'predix-asset': predix.admin.asset.Asset,
            'predix-blobstore': predix.admin.blobstore.BlobStore,
            'predix-timeseries': predix.admin.timeseries.TimeSeries,
            'predix-weather': predix.admin.weather.WeatherForecast,
            'logstash': predix.admin.logstash.Logging,
        }

    def create_manifest_from_space(self):
        """
        Populate a manifest file generated from details from the
        cloud foundry space environment.
        """
        space = predix.admin.cf.spaces.Space()

        summary = space.get_space_summary()
        for instance in summary['services']:
            service_type = instance['service_plan']['service']['label']
            name = instance['name']
            if service_type == 'predix-uaa':
                uaa = predix.admin.uaa.UserAccountAuthentication(name=name)
                uaa.add_to_manifest(self)
            elif service_type == 'predix-acs':
                acs = predix.admin.acs.AccessControl(name=name)
                acs.add_to_manifest(self)
            elif service_type == 'predix-asset':
                asset = predix.admin.asset.Asset(name=name)
                asset.add_to_manifest(self)
            elif service_type == 'predix-timeseries':
                timeseries = predix.admin.timeseries.TimeSeries(name=name)
                timeseries.add_to_manifest(self)
            elif service_type == 'predix-blobstore':
                blobstore = predix.admin.blobstore.BlobStore(name=name)
                blobstore.add_to_manifest(self)
            elif service_type == 'us-weather-forecast':
                weather = predix.admin.weather.WeatherForecast(name=name)
                weather.add_to_manifest(self)
            else:
                logging.warn("Unsupported service type: %s" % service_type)

    def lock_to_org_space(self):
        """
        Lock the manifest to the current organization and space regardless of
        Cloud Foundry target.
        """
        self.add_env_var('PREDIX_ORGANIZATION_GUID', self.space.org.guid)
        self.add_env_var('PREDIX_ORGANIZATION_NAME', self.space.org.name)
        self.add_env_var('PREDIX_SPACE_GUID', self.space.guid)
        self.add_env_var('PREDIX_SPACE_NAME', self.space.name)
        self.write_manifest()

    def create_uaa(self, admin_secret):
        """
        Creates an instance of UAA Service.

        :param admin_secret: The secret password for administering the service
            such as adding clients and users.
        """
        uaa = predix.admin.uaa.UserAccountAuthentication()
        if not uaa.exists():
            uaa.create(admin_secret)
            uaa.add_to_manifest(self)
        return uaa

    def create_client(self, client_id, client_secret):
        """
        Create a client and add it to the manifest.

        :param client_id: The client id used to authenticate as a client
            in UAA.

        :param client_secret: The secret password used by a client to
            authenticate and generate a UAA token.
        """
        uaa = predix.admin.uaa.UserAccountAuthentication()
        uaa.create_client(client_id, client_secret)
        uaa.add_client_to_manifest(client_id, client_secret, self)

    def create_timeseries(self):
        """
        Creates an instance of the Time Series Service.
        """
        ts = predix.admin.timeseries.TimeSeries()
        ts.create()

        client_id = self.get_client_id()
        if client_id:
            ts.grant_client(client_id)

        ts.add_to_manifest(self)
        return ts

    def create_asset(self):
        """
        Creates an instance of the Asset Service.
        """
        asset = predix.admin.asset.Asset()
        asset.create()

        client_id = self.get_client_id()
        if client_id:
            asset.grant_client(client_id)

        asset.add_to_manifest(self)
        return asset

    def create_acs(self):
        """
        Creates an instance of the Asset Service.
        """
        acs = predix.admin.acs.AccessControl()
        acs.create()

        client_id = self.get_client_id()
        if client_id:
            acs.grant_client(client_id)

        acs.grant_client(client_id)
        acs.add_to_manifest(self)
        return acs

    def create_weather(self):
        """
        Creates an instance of the Asset Service.
        """
        weather = predix.admin.weather.WeatherForecast()
        weather.create()

        client_id = self.get_client_id()
        if client_id:
            weather.grant_client(client_id)

        weather.grant_client(client_id)
        weather.add_to_manifest(self)
        return weather

    def create_blobstore(self):
        """
        Creates an instance of the BlobStore Service.
        """
        blobstore = predix.admin.blobstore.BlobStore()
        blobstore.create()

        blobstore.add_to_manifest(self)
        return blobstore

    def create_logstash(self):
        """
        Creates an instance of the Logging Service.
        """
        logstash = predix.admin.logstash.Logging()
        logstash.create()
        logstash.add_to_manifest(self)

        logging.info('Install Kibana-Me-Logs application by following GitHub instructions')
        logging.info('git clone https://github.com/cloudfoundry-community/kibana-me-logs.git')

        return logstash

    def get_service_marketplace(self, available=True, unavailable=False,
            deprecated=False):
        """
        Returns a list of service names.  Can return all services, just
        those supported by PredixPy, or just those not yet supported by
        PredixPy.

        :param available: Return the services that are
            available in PredixPy.  (Defaults to True)

        :param unavailable: Return the services that are not yet
            supported by PredixPy.  (Defaults to False)

        :param deprecated: Return the services that are
            supported by PredixPy but no longer available. (True)
        """

        supported = set(self.supported.keys())
        all_services = set(self.space.get_services())

        results = set()
        if available:
            results.update(supported)
        if unavailable:
            results.update(all_services.difference(supported))
        if deprecated:
            results.update(supported.difference(all_services))

        return list(results)




import logging

import predix.admin.cf.api
import predix.admin.cf.orgs
import predix.admin.cf.apps
import predix.admin.cf.services


class Space(object):
    """
    Operations and data for Cloud Foundry Spaces.
    """
    def __init__(self, *args, **kwargs):
        super(Space, self).__init__(*args, **kwargs)

        self.api = predix.admin.cf.api.API()

        self.name = self.api.config.get_space_name()
        self.guid = self.api.config.get_space_guid()

        self.org = predix.admin.cf.orgs.Org()

    def _get_spaces(self):
        """
        Get the marketplace services.
        """
        guid = self.api.config.get_organization_guid()
        uri = '/v2/organizations/%s/spaces' % (guid)
        return self.api.get(uri)

    def get_spaces(self):
        """
        Return a flat list of the names for spaces in the organization.
        """
        self.spaces = []
        for resource in self._get_spaces()['resources']:
            self.spaces.append(resource['entity']['name'])

        return self.spaces

    def get_space_services(self):
        """
        Returns the services available for use in the space.  This may
        not always be the same as the full marketplace.
        """
        uri = '/v2/spaces/%s/services' % (self.guid)
        return self.api.get(uri)

    def create_space(self, space_name):
        """
        Create a new space of the given name.
        """
        body = {
            'name': space_name,
            'organization_guid': self.api.config.get_organization_guid()
        }
        return self.api.post('/v2/spaces', body)

    def delete_space(self, space_name):
        """
        Delete a space of the given name.
        """
        return self.api.delete("/v2/spaces/%s" % (self.guid))

    def get_space_summary(self):
        """
        Returns a summary of apps and services within a given
        cloud foundry space.

        It is the call used by `cf s` or `cf a` for quicker
        responses.
        """
        uri = '/v2/spaces/%s/summary' % (self.guid)
        return self.api.get(uri)

    def _get_apps(self):
        """
        Returns raw results for all apps in the space.
        """
        uri = '/v2/spaces/%s/apps' % (self.guid)
        return self.api.get(uri)

    def get_apps(self):
        """
        Returns a list of all of the apps in the space.
        """
        apps = []
        for resource in self._get_apps()['resources']:
            apps.append(resource['entity']['name'])

        return apps

    def has_app(self, app_name):
        """
        Simple test to see if we have a name conflict
        for the application.
        """
        return app_name in self.get_apps()

    def _get_services(self):
        """
        Return the available services for this space.
        """
        uri = '/v2/spaces/%s/services' % (self.guid)
        return self.api.get(uri)

    def get_services(self):
        """
        Returns a flat list of the service names available
        from the marketplace for this space.
        """
        services = []
        for resource in self._get_services()['resources']:
            services.append(resource['entity']['label'])

        return services

    def _get_instances(self):
        """
        Returns the service instances activated in this space.
        """
        uri = '/v2/spaces/%s/service_instances' % (self.guid)
        return self.api.get(uri)

    def get_instances(self):
        """
        Returns a flat list of the names of services created
        in this space.
        """
        services = []
        for resource in self._get_instances()['resources']:
            services.append(resource['entity']['name'])

        return services

    def has_service_with_name(self, service_name):
        """
        Tests whether a service with the given name exists in
        this space.
        """
        return service_name in self.get_instances()

    def has_service_of_type(self, service_type):
        """
        Tests whether a service instance exists for the given
        service.
        """
        summary = self.get_space_summary()
        for instance in summary['services']:
            if service_type == instance['service_plan']['service']['label']:
                return True

        return False

    def purge(self):
        """
        Remove all services and apps from the space.

        Will leave the space itself, call delete_space() if you
        want to remove that too.

        Similar to `cf delete-space -f <space-name>`.
        """
        logging.warn("Purging all services from space %s" %
                (self.name))

        service = predix.admin.cf.services.Service()
        for service_name in self.get_instances():
            service.purge(service_name)

        apps = predix.admin.cf.apps.App()
        for app_name in self.get_apps():
            apps.delete_app(app_name)




import os
import re
import time
import json
import logging
import datetime
import websocket

import predix.config
import predix.service


class TimeSeries(object):
    """
    Client library for working with the Time Series service.

    Learn more about Predix Time Series:
    https://www.predix.io/services/service.html?id=1177

    """
    # "Constants" for data quality values
    BAD = 0
    UNCERTAIN = 1
    NA = 2
    GOOD = 3

    def __init__(self, read=True, write=True, *args, **kwargs):
        """
        Time Series by default will grant the client both read
        and write permissions.  Either can be disabled.
        """
        super(TimeSeries, self).__init__(*args, **kwargs)

        self.zone_id = None
        if read:
            key = predix.config.get_env_key(self, 'query_zone_id')
            self.query_zone_id = os.environ.get(key)
            if not self.query_zone_id:
                raise ValueError("%s env unset" % key)

            key = predix.config.get_env_key(self, 'query_uri')
            self.query_uri = os.environ.get(key)
            if not self.query_uri:
                raise ValueError("%s environment unset" % key)

            self.zone_id = self.query_zone_id

        if write:
            key = predix.config.get_env_key(self, 'ingest_zone_id')
            self.ingest_zone_id = os.environ.get(key)
            if not self.ingest_zone_id:
                raise ValueError("%s env unset" % key)

            key = predix.config.get_env_key(self, 'ingest_uri')
            self.ingest_uri = os.environ.get(key)
            if not self.ingest_uri:
                raise ValueError("%s environment unset" % key)

            self.zone_id = self.ingest_zone_id

        self.service = predix.service.Service(self.zone_id)

        # Store a websocket connection once opened
        self.ws = None

        # Store and forward any datapoints as a single transaction
        self._queue = []

    def __del__(self):
        """
        Destructor to make sure an open websocket connection is closed.
        """
        # No need to delete if not properly initialized
        if not hasattr(self, '_queue'):
            return

        if len(self._queue) > 0:
            logging.warn("%s buffered datapoints in queue lost." %
                    (len(self._queue)))

        if self.ws:
            self.ws.close()

    def authenticate_as_client(self, client_id, client_secret):
        """
        Will authenticate for the given client / secret.
        """
        self.service.uaa.authenticate(client_id, client_secret)

    def _get_aggregations(self):
        """
        Returns all of the aggregations in time series.  There is no support
        in the service for filtering or paginations.
        """
        url = self.query_uri + '/v1/aggregations'
        return self.service._get(url)

    def get_aggregations(self):
        """
        Returns all of the aggregations that can be used with the
        Time Series Service.
        """
        return self._get_aggregations()['results']

    def _get_tags(self):
        """
        Returns all of the tags in time series.  There is no support
        in the service for filtering or paginations.
        """
        return self.service._get(self.query_uri + '/v1/tags')

    def get_tags(self):
        """
        Returns a list of the tag names in the timeseries service
        instance.
        """
        return self._get_tags()['results']

    def _get_datapoints(self, params):
        """
        Will make a direct REST call with the given json body payload to
        get datapoints.
        """
        url = self.query_uri + '/v1/datapoints'
        return self.service._get(url, params=params)

    def _post_datapoints(self, body):
        url = self.query_uri + '/v1/datapoints'
        return self.service._post(url, body)

    def get_values(self, *args, **kwargs):
        """
        Convenience method that for simple single tag queries will
        return just the values to be iterated on.
        """
        if isinstance(args[0], list):
            raise ValueError("Can only get_values() for a single tag.")

        response = self.get_datapoints(*args, **kwargs)
        for value in response['tags'][0]['results'][0]['values']:
            yield [datetime.datetime.utcfromtimestamp(value[0]/1000),
                   value[1],
                   value[2]]

    def is_good(self, quality):
        """
        Simple test if data quality is GOOD.
        """
        return quality == ts.GOOD

    def is_bad(self, quality):
        """
        Simple test if data quality value is BAD.
        """
        return quality == ts.BAD

    def get_datapoints(self, tags, start=None, end=None, order=None,
            limit=None, qualities=None, attributes=None, measurement=None,
            aggregations=None, post=False):
        """
        Returns all of the datapoints that match the given query.

            - tags: list or string identifying the name/tag (ie. "temp")
            - start: data after this, absolute or relative (ie. '1w-ago' or
              1494015972386)
            - end: data before this value
            - order: ascending (asc) or descending (desc)
            - limit: only return a few values (ie. 25)
            - qualities: data quality value (ie. [ts.GOOD, ts.UNCERTAIN])
            - attributes: data attributes (ie. {'unit': 'mph'})
            - measurement: tuple of operation and value (ie. ('gt', 30))
            - aggregations: summary statistics on data results (ie. 'avg')
            - post: POST query instead of GET (caching implication)

        A few additional observations:
            - allow service to do most data validation
            - order is applied before limit so resultset will differ

        The returned results match what the service response is so you'll
        need to unpack it as appropriate.  Oftentimes what you want for
        a simple single tag query will be:

            response['tags'][0]['results'][0]['values']

        """
        params = {}

        # Documentation says start is required for GET but not POST, but
        # seems to be required all the time, so using sensible default.
        if not start:
            start = '1w-ago'
            logging.info("Defaulting to timeseries data since %s" % (start))

        # Start date can be absolute or relative, only certain legal values
        # but service will throw error if used improperly.  (ms, s, mi, h, d,
        # w, mm, y).  Relative dates must end in -ago.
        params['start'] = start

        # Docs say when making POST with a start that end must also be
        # specified, but this does not seem to be the case.
        if end:
            params['end'] = end

        params['tags'] = []
        if not isinstance(tags, list):
            tags = [tags]

        for tag in tags:
            query = {}
            query['name'] = tag

            # Limit resultset with an integer value
            if limit:
                query['limit'] = int(limit)

            # Order must be 'asc' or 'desc' but will get sensible error
            # from service.
            if order:
                query['order'] = order

            # Filters are complex and support filtering by
            # quality, measurement, and attributes.
            filters = {}

            # Check for the quality of the datapoints
            if qualities is not None:
                if isinstance(qualities, int) or isinstance(qualities, str):
                    qualities = [qualities]

                # Timeseries expects quality to be a string, not integer,
                # so coerce each into a string
                for i, quality in enumerate(qualities):
                    qualities[i] = str(quality)

                filters['qualities'] = {"values": qualities}

            # Check for attributes on the datapoints, expected to be
            # a dictionary of key / value pairs that datapoints must match.
            if attributes is not None:
                if not isinstance(attributes, dict):
                    raise ValueError("Attribute filters must be dictionary.")

                filters['attributes'] = attributes

            # Check for measurements that meets a given comparison operation
            # such as ge, gt, eq, ne, le, lt
            if measurement is not None:
                filters['measurements'] = {
                        'condition': measurement[0],
                        'values': measurement[1]
                        }

            # If we found any filters add them to the query
            if filters:
                query['filters'] = filters

            # Handle any additional aggregations of dataset
            if aggregations is not None:
                if not isinstance(aggregations, list):
                    aggregations = [aggregations]

                query['aggregations'] = []
                for aggregation in aggregations:
                    query['aggregations'].append({
                        'sampling': {'datapoints': 1},
                        'type': aggregation })

            params['tags'].append(query)

        if post:
            return self._post_datapoints(params)
        else:
            return self._get_datapoints({"query": json.dumps(params)})

    def _get_latest(self, params):
        """
        Will directly pass the given params as parameters
        to the GET endpoint.
        """
        uri = self.query_uri + '/v1/datapoints/latest'
        return self.service._get(uri, params=params)

    def _post_latest(self, body):
        """
        Will directly pass the given body object as the JSON
        payload to a POST call.
        """
        uri = self.query_uri + '/v1/datapoints/latest'
        return self.service._post(uri, body)

    def get_latest(self, tags, post=False):
        """
        Similar to get_datapoints() but will only return the very
        last datapoint ingested for the given tag or tags (if a list).

        If given the optional post parameter will make the call as a POST
        rather than a GET request.  The GET can have an advantage of url
        based response caching and the POST has the advantage of a larger
        and more complex query.
        """
        params = {}
        params['tags'] = tags

        if post:
            return self._post_latest(params)
        else:
            return self._get_latest(params)

    def _create_connection(self):
        """
        Create a new websocket connection with proper headers.
        """
        logging.debug("Initializing new websocket connection.")
        headers = {
            'Authorization': self.service._get_bearer_token(),
            'Predix-Zone-Id': self.ingest_zone_id,
            'Content-Type': 'application/json',
        }
        url = self.ingest_uri

        logging.debug("URL=" + str(url))
        logging.debug("HEADERS=" + str(headers))

        # Should consider connection pooling and longer timeouts
        return websocket.create_connection(url, header=headers)

    def _get_websocket(self, reuse=True):
        """
        Reuse existing connection or create a new connection.
        """
        # Check if still connected
        if self.ws and reuse:
            if self.ws.connected:
                return self.ws

            logging.debug("Stale connection, reconnecting.")

        self.ws = self._create_connection()
        return self.ws

    def _send_to_timeseries(self, message):
        """
        Establish or reuse socket connection and send
        the given message to the timeseries service.
        """
        logging.debug("MESSAGE=" + str(message))

        result = None
        try:
            ws = self._get_websocket()
            ws.send(json.dumps(message))
            result = ws.recv()
        except (websocket.WebSocketConnectionClosedException, Exception) as e:
            logging.debug("Connection failed, will try again.")
            logging.debug(e)

            ws = self._get_websocket(reuse=False)
            ws.send(json.dumps(message))
            result = ws.recv()

        logging.debug("RESULT=" + str(result))
        return result

    def queue(self, name, value, quality=None, timestamp=None,
            attributes=None):
        """
        To reduce network traffic, you can buffer datapoints and
        then flush() anything in the queue.
        """
        # Get timestamp first in case delay opening websocket connection
        # and it must have millisecond accuracy
        if not timestamp:
            timestamp = int(round(time.time() * 1000))

        # Only specific quality values supported
        if quality not in [self.BAD, self.GOOD, self.NA, self.UNCERTAIN]:
            quality = self.UNCERTAIN

        # Check if adding to queue of an existing tag and add second datapoint
        for point in self._queue:
            if point['name'] == name:
                point['datapoints'].append([timestamp, value, quality])
                return

        # If adding new tag, initialize and set any attributes
        datapoint = {
            "name": name,
            "datapoints": [[timestamp, value, quality]]
        }

        # Attributes are specified for datapoint
        if attributes:
            # Validate rules for attribute keys to provide guidance.
            invalid_value = ':;= '
            has_invalid_value = re.compile(r'[%s]' % (invalid_value)).search
            has_valid_key = re.compile(r'^[\w\.\/\-]+$').search

            for (key, val) in attributes.items():
                # Values cannot be NULL
                if not val:
                    raise ValueError("Attribute (%s) must have value." % (key))

                # Values cannot contain certain arbitrary characters
                if bool(has_invalid_value(val)):
                    raise ValueError("Attribute (%s) cannot contain (%s)." %
                            (key, invalid_value))

                # Attributes have to be alphanumeric-ish
                if not bool(has_valid_key):
                    raise ValueError("Key (%s) not alphanumeric-ish." % (key))

            datapoint['attributes'] = attributes

        self._queue.append(datapoint)
        logging.debug("QUEUE: " + str(len(self._queue)))

    def send(self, name=None, value=None, **kwargs):
        """
        Can accept a name/tag and value to be queued and then send anything in
        the queue to the time series service.  Optional parameters include
        setting quality, timestamp, or attributes.

        Example of sending a batch of values:

            queue('temp', 70.1)
            queue('humidity', 20.4)
            send()

        Example of sending one and flushing queue immediately

            send('temp', 70.3)
            send('temp', 70.4, quality=ts.GOOD, attributes={'unit': 'F'})

        """
        if name and value:
            self.queue(name, value, **kwargs)

        timestamp = int(round(time.time() * 1000))

        # The label "name" or "tag" is sometimes used ambiguously
        msg = {
            "messageId": timestamp,
            "body": self._queue
        }

        self._queue = []

        return self._send_to_timeseries(msg)




import os

import predix.security.uaa
import predix.admin.service

class ParkingPlanning(object):
    """
    Optimize operations and planning with vehicle parking data.
    """
    def __init__(self, name=None, uaa=None, *args, **kwargs):
        super(ParkingPlanning, self).__init__(*args, **kwargs)
        self.service_name = 'ie-traffic'
        self.plan_name = 'Beta'
        self.service = predix.admin.service.PredixService(self.service_name,
                self.plan_name, name=name, uaa=uaa)

    def exists(self):
        """
        Returns whether or not this service already exists.
        """
        return self.service.exists()

    def create(self):
        """
        Create an instance of the Parking Planning Service with the
        typical starting settings.
        """
        self.service.create()
        os.environ[self.__module__ + '.uri'] = self.service.settings.data['url']
        os.environ[self.__module__ + '.zone_id'] = self.get_predix_zone_id()

    def grant_client(self, client_id):
        """
        Grant the given client id all the scopes and authorities
        needed to work with the parking planning service.
        """
        zone = self.get_oauth_scope()
        scopes = ['openid', zone]
        authorities = ['uaa.resource', zone]

        self.service.uaa.uaac.update_client_grants(client_id, scope=scopes,
                authorities=authorities)

        return self.service.uaa.uaac.get_client(client_id)

    def get_oauth_scope(self):
        """
        Simply returns the configured service oauth scope needed
        in client uaa grants.
        """
        return self.service.settings.data['zone']['oauth-scope']

    def get_predix_zone_id(self):
        """
        Simply returns the configured service predix zone id.
        """
        return self.service.settings.data['zone']['http-header-value']

    def add_to_manifest(self, manifest):
        """
        Add useful details to the manifest about this service
        so that it can be used in an application.

        :param manifest: An predix.admin.app.Manifest object
            instance that manages reading/writing manifest config
            for a cloud foundry app.
        """
        # Add this service to list of services
        manifest.add_service(self.service.name)

        # Add environment variables
        manifest.add_env_var(self.__module__ + '.uri',
                self.service.settings.data['url'])
        manifest.add_env_var(self.__module__ + '.zone_id',
                self.get_predix_zone_id())

        manifest.write_manifest()




import os
import json
import uuid
import logging
import requests

import predix.config
import predix.service
import predix.security.uaa


class Asset(object):
    """
    Client library for working with the Predix Asset Service.  For more details
    on use of the service please see official docs:
    https://www.predix.io/services/service.html?id=1171
    """
    def __init__(self, *args, **kwargs):
        super(Asset, self).__init__(*args, **kwargs)

        key = predix.config.get_env_key(self, 'zone_id')
        self.zone_id = os.environ.get(key)
        if not self.zone_id:
            raise ValueError("%s environment unset" % key)

        key = predix.config.get_env_key(self, 'uri')
        self.uri = os.environ.get(key)
        if not self.uri:
            raise ValueError("%s environment unset" % key)

        self.service = predix.service.Service(self.zone_id)

    def authenticate_as_client(self, client_id, client_secret):
        """
        Will authenticate for the given client / secret.
        """
        self.service.uaa.authenticate(client_id, client_secret)

    def _get_collections(self):
        """
        Returns the names of all user-defined domain object collections with
        counts for number of domain objects contained in that collection.

        ..

            [ { "collection": "volcano", "count": 1 }, ... ]

        """
        uri = self.uri
        return self.service._get(uri)

    def get_collections(self):
        """
        Returns a flat list of the names of collections in the asset
        service.

        ..

            ['wind-turbines', 'jet-engines']

        """
        collections = []
        for result in self._get_collections():
            collections.append(result['collection'])

        return collections

    def get_collection(self, collection, filter=None, fields=None,
            page_size=None):
        """
        Returns a specific collection from the asset service with
        the given collection endpoint.

        Supports passing through parameters such as...
        - filters such as "name=Vesuvius" following GEL spec
        - fields such as "uri,description" comma delimited
        - page_size such as "100" (the default)

        """
        params = {}
        if filter:
            params['filter'] = filter
        if fields:
            params['fields'] = fields
        if page_size:
            params['pageSize'] = page_size

        uri = self.uri + '/v1' + collection
        return self.service._get(uri, params=params)

    def create_guid(self, collection=None):
        """
        Returns a new guid for use in posting a new asset to a collection.
        """
        guid = str(uuid.uuid4())
        if collection:
            return str.join('/', [collection, guid])
        else:
            return guid

    def post_collection(self, collection, body):
        """
        Creates a new collection.  This is mostly just transport layer
        and passes collection and body along.  It presumes the body
        already has generated.

        The collection is *not* expected to have the id.
        """
        assert isinstance(body, (list)), "POST requires body to be a list"
        assert collection.startswith('/'), "Collections must start with /"
        uri = self.uri + '/v1' + collection
        return self.service._post(uri, body)

    def put_collection(self, collection, body):
        """
        Updates an existing collection.

        The collection being updated *is* expected to include the id.
        """

        uri = self.uri + '/v1' + collection
        return self.service._put(uri, body)

    def delete_collection(self, collection):
        """
        Deletes an existing collection.

        The collection being updated *is* expected to include the id.
        """
        uri = str.join('/', [self.uri, collection])
        return self.service._delete(uri)

    def patch_collection(self, collection, changes):
        """
        Will make specific updates to a record based on JSON Patch
        documentation.

            https://tools.ietf.org/html/rfc6902

        the format of changes is something like::

            [{
                'op': 'add',
                'path': '/newfield',
                'value': 'just added'
            }]

        """
        uri = str.join('/', [self.uri, collection])
        return self.service._patch(uri, changes)

    def get_audit(self):
        """
        Return audit report for asset.  Disabled by default.
        """
        return self.service._get(self.uri + '/v1/system/audit')

    def get_audit_changes(self):
        """
        Return change log for audit.  Disabled by default.
        """
        return self.service._get(self.uri + '/v1/system/audit/changes')

    def get_audit_snapshots(self):
        """
        Return an audit snapshot.  Disabled by default.
        """
        return self.service._get(self.uri + '/v1/system/audit/snapshots')

    def get_scripts(self):
        """
        Return any configured scripts for asset service.
        """
        return self.service._get(self.uri + '/v1/system/scripts')

    def get_messages(self):
        """
        Return any system messages related to asset systems.
        """
        return self.service._get(self.uri + '/v1/system/messages')

    def get_configs(self):
        """
        Return the configuration for the asset service.
        """
        return self.service._get(self.uri + '/v1/system/configs')

    def get_triggers(self):
        """
        Return configured triggers in the asset system.
        """
        return self.service._get(self.uri + '/v1/system/triggers')

    def save(self, collection):
        """
        Save an asset collection to the service.
        """
        assert isinstance(collection, predix.data.asset.AssetCollection), "Expected AssetCollection"
        collection.validate()
        self.put_collection(collection.uri, collection.__dict__) # MAINT: no

class AssetCollection(object):
    """
    User Defined Domain Objects are the customizable collections to represent
    data in the Asset Service.

    This is experimental to provide a base class for a sort of ORM between
    domain objects to marshall and unmarshall between Python and the REST
    endpoints.
    """
    def __init__(self, parent=None, guid=None, *args, **kwargs):
        super(AssetCollection, self).__init__(*args, **kwargs)

        # You have the right to a guid, if you cannot afford a guid...
        if not guid:
            guid = str(uuid.uuid4())

        # By naming collection after classname we get safe URI
        # naming rules as well.
        collection = self.get_collection()

        # There is a no more than 2 forward slash limitation for uri, so
        # collections cannot really be nested deeper than one level.
        self.uri = '/' + str.join('/', [collection, guid])

    def __repr__(self):
        return json.dumps(self.__dict__)

    def __str__(self):
        return json.dumps(self.__dict__)

    def get_collection(self):
        return type(self).__name__.lower()

    def validate(self):
        """
        If an asset collection wants any client-side validation the
        object can override this method and it is called anytime
        we're saving.
        """
        return




from setuptools import setup, find_packages

install_requires = [
        "python-dateutil",
        "PyYAML",
        "requests",
        "boto3",
        "websocket",
        "websocket-client"
    ]

setup_requires = [
    ]

long_description = 'See GitHub README.md for more details.'
with open('README.md') as file:
    long_description = file.read()

setup(
        name="predix",
        version="0.0.9",
        author="Jayson DeLancey",
        author_email="jayson.delancey@ge.com",
        description="Python Client SDK for Predix Services",
        long_description=long_description,
        setup_requires=setup_requires,
        install_requires=install_requires,
        package_data={},
        packages=find_packages(exclude=['test', 'test.*']),
        test_suite="test",
        entry_points={
            'console_scripts': [
                ]
        },
        tests_require=[],
        keywords=['predix', 'ge', 'asset', 'analytics'],
        url="https://github.com/PredixDev/predixpy",
        classifiers=[
            'Intended Audience :: Developers',
            'Natural Language :: English',
            'Programming Language :: Python :: 2.7',
            'License :: OSI Approved :: BSD License',
            'Development Status :: 3 - Alpha']
    )




import os
import boto3
import logging

import predix.service
import predix.config


class BlobStore(object):
    """
    .. important::

       This service will only work from the Predix Cloud -- Firewall will block
       any traffic not originating from within the Predix environment.

    The BlobStore is the place to store Binary Large Objects, ie. files that
    could be images, csv files, cad files, pdf files, etc.

    Underlying BlobStore is the AWS S3 service so you will need to be familiar
    with the boto3 library from AWS to learn how to work with buckets.  A few
    methods are provided for common patterns you can use for reference.
    """
    def __init__(self):

        instance = os.environ.get('CF_INSTANCE_ADDR')
        if not instance:
            raise ValueError("This service can only be used in the Predix Cloud Foundry environment.")

        host = predix.config.get_env_key(self, 'host')
        self.host = os.environ.get(host)
        if not self.host:
            raise ValueError("%s environment unset" % host)

        # Protocol may not be specified in host path
        if 'https://' not in self.host:
            self.host = 'https://' + self.host

        access_key_id = predix.config.get_env_key(self, 'access_key_id')
        self.access_key_id = os.environ.get(access_key_id)
        if not self.access_key_id:
            raise ValueError("%s environment unset" % access_key_id)

        secret_access_key = predix.config.get_env_key(self,
                'secret_access_key')
        self.secret_access_key = os.environ.get(secret_access_key)
        if not self.secret_access_key:
            raise ValueError("%s environment unset" % secret_access_key)

        bucket_name = predix.config.get_env_key(self, 'bucket_name')
        self.bucket_name = os.environ.get(bucket_name)
        if not self.bucket_name:
            raise ValueError("%s environment unset" % bucket_name)

        self.session = boto3.session.Session(
                aws_access_key_id=self.access_key_id,
                aws_secret_access_key=self.secret_access_key)
        config = boto3.session.Config(signature_version='s3', s3={
                'addressing_style': 'virtual'})
        self.client = self.session.client('s3', endpoint_url=self.host,
                config=config)

    def list_buckets(self, *args, **kwargs):
        """
        This method is primarily for illustration and just calls the boto3
        client implementation of list_buckets directly but is a common task for
        first time Predix BlobStore users.
        """
        return self.client.list_buckets(**kwargs)

    def list_objects(self, bucket_name=None, **kwargs):
        """
        This method is primarily for illustration and just calls the 
        boto3 client implementation of list_objects but is a common task
        for first time Predix BlobStore users.
        """
        if not bucket_name: bucket_name = self.bucket_name
        return self.client.list_objects(Bucket=bucket_name, **kwargs)

    def upload_file(self, src_filepath, dest_filename=None, bucket_name=None,
            **kwargs):
        """
        This method is primarily for illustration and just calls the 
        boto3 client implementation of upload_file but is a common task
        for first time Predix BlobStore users.
        """
        if not bucket_name: bucket_name = self.bucket_name
        if not dest_filename: dest_filename = src_filepath
        return self.client.upload_file(src_filepath, bucket_name,
                dest_filename, **kwargs)
